{------------------------------------------------------------------}
{  TDBEditor version 0.6 beta 1                                    }         
{                                                                  }   
{    Copyright  1997, 1998 - All Rights Reserved.                 }  
{    Developed By: Sun Yiyi                                        }
{    E-MAIL: yysun@nt4.gislab.ecnu.edu.cn                          }
{    http://www..gislab.ecnu.edu.cn                                }
{                                                                  }
{------------------------------------------------------------------}

unit DBEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, DB, DBTables, DBCtrls;
type
  TDBEditor = class;

  TDBEditorLink = class(TDataLink)
  private
    FDBEditor: TDBEditor;
  protected
    procedure ActiveChanged; override;
  public
    constructor Create(ADBEditor: TDBEditor);
  end;

  TDBEditor = class(TScrollBox)
  private
    UpdatingControls: boolean;
    FMinWidth: integer;
    FAutoSize: boolean;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
  protected
    FDataLink: TDBEditorLink;
    FControlList: TStrings;
    procedure ReCreateControls; virtual;
    procedure ReOrderControls;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ActiveChanged;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property AutoSize: boolean read FAutoSize write FAutoSize;
    property MinWidth: integer read FMinWidth write FMinWidth;
  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('My DBTools', [TDBEditor]);
end;

{---------------------------------------------------------------------------}
constructor TDBEditorLink.Create(ADBEditor: TDBEditor);
begin
   inherited Create;
   FDBEditor := ADBEditor;
end;

procedure TDBEditorLink.ActiveChanged;
begin
   inherited;
   FDBEditor.ActiveChanged;
end;

{---------------------------------------------------------------------------}
constructor TDBEditor.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);     UpdatingControls := false;
   FControlList := TStringList.Create;
   FDataLink := TDBEditorLink.Create(self);
   FAutoSize := False;
   FMinWidth := 100;
end;

destructor TDBEditor.Destroy;
begin
   FControlList.Free;
   FDataLink.Free;
   inherited Destroy;
end;

function TDBEditor.GetDataSource: TDataSource;
begin
   Result := FDataLink.DataSource;
end;

procedure TDBEditor.SetDataSource(Value: TDataSource);
begin
   FDataLink.DataSource := Value;
end;

procedure TDBEditor.ActiveChanged;
var ii: integer;
begin
   visible := false;
   for ii := FControlList.Count - 1 Downto 0 do begin
       FControlList[ii] := 'D'+InttoStr(ii);
   end;
   if FDataLink.Active and
       Assigned(FDataLink.DataSource) and FDataLink.Dataset.Active then begin
       UpdatingControls:=true;
       ReCreateControls;
       UpdatingControls:=false;
       ReOrderControls;
   end;
   visible := true;
end;

procedure TDBEditor.ReCreateControls;
var jj: integer;
    Newlabel : TLabel;
    NewDBEdit : TDBEdit;
begin
   if not Assigned(Datasource) then exit;
   try
   for jj := 0 to FDataLink.DataSet.FieldCount-1 do
   if Datasource.DataSet.Fields[jj].visible then begin
       Newlabel := Tlabel.create(self);
       Newlabel.Parent := self;
       Newlabel.Caption:= Datasource.DataSet.Fields[jj].DisplayLabel+' ';
       NewDBEdit := TDBEdit.Create(self);
       NewDBEdit.Parent := self;
{
       NewDBEdit.Name := Datasource.DataSet.Fields[jj].Name;
       NewDBEdit.Text := '';
}
       NewDBEdit.DataField:= Datasource.DataSet.Fields[jj].FieldName;
       NewDBEdit.Datasource:= Datasource;
       FControlList.AddObject('L'+InttoStr(jj), Newlabel);
       FControlList.AddObject('E'+InttoStr(jj), NewDBEdit);
   end;
   except
   end;
end;

procedure TDBEditor.ReOrderControls;
var ii, jj, ww, hh : integer;
    Newlabel : TLabel;
    NewDBEdit : TDBEdit;
begin
   if UpdatingControls then exit;
   Visible := false;
   DisableAutoRange;
   for jj := FControlList.Count - 1 downto 0 do
   if pos('D', FControlList[jj])>0 then begin
       RemoveControl(TControl(FControlList.Objects[jj]));
       FControlList.Objects[jj].free;
       FControlList.Delete(jj);
   end;
   EnableAutoRange;
   visible := true;
   if (FControlList.Count =0)then exit;
   Visible := false;
   hh := 12;    ww := Width div 5;
   if ww < FMinWidth then ww := FMinWidth;
   ii := 0;    HorzScrollBar.Position := 0;    VertScrollBar.Position := 0;
   for jj := 0 to FControlList.Count - 1 do
   if pos('L', FControlList[jj])>0 then begin
       Newlabel := TLabel(FControlList.Objects[jj]);
       if (ii mod 2) = 0 then Newlabel.SetBounds(ww-Newlabel.width, (ii+1)*hh+4, Newlabel.width, Newlabel.height)
                         else Newlabel.SetBounds(ww*3-Newlabel.width, ii*hh+4, Newlabel.width, Newlabel.height);
       inc(ii);
   end;
   ii := 0;
   for jj := 0 to FControlList.Count - 1 do
   if pos('E', FControlList[jj])>0 then begin
       NewDBEdit := TDBEdit(FControlList.Objects[jj]);
       if (ii mod 2) = 0 then NewDBEdit.SetBounds(ww, (ii+1)*hh, ww, NewDBEdit.height)
                         else NewDBEdit.SetBounds(ww*3, ii*hh, ww, NewDBEdit.height);
       inc(ii);
   end;
   if FAutoSize then begin
       Height := (ii+3)*hh;
       width := 5 * ww;
   end;
{
   else begin
       HorzScrollBar.Range := HorzScrollBar.Range - Width;
       VertScrollBar.Range := VertScrollBar.Range - Height;
   end;
}
   visible := true;
end;

procedure TDBEditor.Resize;
begin
   inherited;
   ReOrderControls;
end;

end.
